<?php
declare(strict_types=1);

namespace Ep\TecDoc\Plugin;

use Ak\Wp\Plugin\PluginInterface;
use Ep\TecDoc\TecDocService;

/**
 * Class TecDocProduct
 *
 * @package Ep\TecDoc
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/ep-tecdoc
 */
final class TecDocProduct implements PluginInterface
{
    private TecDocService $tecdoc;

    /**
     * @param TecDocService $tecdoc
     */
    public function __construct(TecDocService $tecdoc)
    {
        $this->tecdoc = $tecdoc;
    }

    public function attach(): void
    {
        add_filter('woocommerce_product_tabs', [$this, 'product_tabs']);
    }

    private function getArticle(int $postId): ?array
    {
        global $wp_query;
        
        if (($tecdocKey = get_post_meta($postId, '_tecdoc_key', true)) && isset($wp_query->tecdocArticles)) {
            foreach ($wp_query->tecdocArticles as $article) {
                if ($article['key'] === $tecdocKey) {
                    return $article;
                }
            }
        }
        
        return null;
    }

    public function product_tabs()
    {
        if ($this->getArticle(get_the_ID())) {
            // Adds the new tab
            $tabs['tecdoc_info'] = [
                'title'    => __('Tehnički Podaci', 'ep-tecdoc'),
                'priority' => 50,
                'callback' => [ $this, 'tab_content' ]
            ];

            $tabs['tecdoc_oenumbers'] = [
                'title' => __('OE Brojevi', 'ep-tecdoc'),
                'priority' => 50,
                'callback' => [$this, 'tab_oenumbers'],
            ];

            $tabs['tecdoc_vehicles'] = [
                'title' => __('Povezana Vozila', 'ep-tecdoc'),
                'priority' => 50,
                'callback' => [$this, 'tab_vehicles'],
            ];
        }

        return $tabs;
    }

    function tab_content()
    {
        $article = $this->getArticle(get_the_ID());

        foreach ($article['attributes'] as $attribute) {
            if (100 === $attribute['id']) {
                $attribute['value'] = self::translateFittingPosition($attribute['value']);
            }

            echo '<p>'. __($attribute['name'], 'ep-tecdoc') .': <strong>'. $attribute['value'] .'</strong></p>';
        }

        foreach ($article['infos'] as $info) {
            echo '<p>'. __($info['name'], 'ep-tecdoc') .': <strong>'. $info['text'] .'</strong></p>';
        }
    }

    function tab_oenumbers()
    {
        $article = $this->getArticle(get_the_ID());
        $oenumbers = [];

        foreach ($article['oenumbers'] as $number) {
            $oenumbers[$number['brand']][] = $number['number'];
        }

        if (count($oenumbers) > 0): ?>
            <h3 class="accordion_title"><?= __('OE Brojevi', 'ep-tecdoc'); ?></h3>

            <div class="accordion" rel="0">
                <?php foreach ($oenumbers as $brand => $numbers): ?>
                    <div class="accordion-item">
                        <a href="#" class="accordion-title plain">
                            <button class="toggle"><i class="icon-angle-down"></i></button>
                            <span><?= $brand; ?></span>
                        </a>

                        <div class="accordion-inner">
                            <ul>
                                <?php foreach ($numbers as $number): ?>
                                    <li>
                                        <a href="<?= esc_attr(home_url(sprintf('?s=%s&post_type=product', preg_replace('/[^a-zA-Z0-9]/', '', $number)))); ?>">
                                            <?= $number; ?>
                                        </a>
                                    </li>
                                <?php endforeach; ?>
                            </ul>
                        </div>
                    </div>
                <?php endforeach; ?>
            </div>
        <?php
        endif;
    }

    public function tab_vehicles()
    {
        $article = $this->getArticle(get_the_ID());
        ?>
        <div class="loader-wrapper" style="position: relative; width: 45px; height: 45px;">
            <div class="loader" data-article-id="<?= $article['id']; ?>"></div>
        </div>
        <?php
    }
    
    static function translateFittingPosition($string)
    {
        $translations = [
            'left' => __('Left', 'ep-tecdoc'),
            'right' => __('Right', 'ep-tecdoc'),
            'front' => __('Front', 'ep-tecdoc'),
            'front axle' => __('Front Axle', 'ep-tecdoc'),
            'front axle left' => __('Front Axle Left', 'ep-tecdoc'),
            'front axle right' => __('Front Axle Right', 'ep-tecdoc'),
            'front axle left and right' => __('Front Axle Left and Right', 'ep-tecdoc'),
            'lower front axle' => __('Lower Front Axle', 'ep-tecdoc'),
            'upper front axle' => __('Upper Front Axle', 'ep-tecdoc'),
            'rear' => __('Rear', 'ep-tecdoc'),
            'rear axle' => __('Rear Axle', 'ep-tecdoc'),
            'rear axle left' => __('Rear Axle Left', 'ep-tecdoc'),
            'rear axle right' => __('Rear Axle Right', 'ep-tecdoc'),
            'rear axle left and right' => __('Rear Axle Left and Right', 'ep-tecdoc'),
            'lower rear axle' => __('Lower Rear Axle', 'ep-tecdoc'),
            'upper rear axle' => __('Upper Rear Axle', 'ep-tecdoc'),
        ];

        return $translations[strtolower($string)] ?? $string;
    }
}